@@ -25,7 +25,7 @@
     */
 
 #include <iostream>
-#include "core.h"
+#include "common.h"
 
 using namespace std;
 using namespace Arts;
