@@ -114,7 +114,11 @@ AudioIOALSA::AudioIOALSA()
 	 */
     m_card = snd_defaults_pcm_card();  //!! need interface !!
     m_device = snd_defaults_pcm_device(); //!!
+#ifdef WORDS_BIGENDIAN
+    m_format = SND_PCM_SFMT_S16_BE;
+#else
     m_format = SND_PCM_SFMT_S16_LE;
+#endif
     m_bufferMode = block;  //block/stream (stream mode doesn't work yet)
 
 	if(m_card >= 0) {
