@@ -35,6 +35,7 @@
 #include <stack>
 #include <vector>
 #include <list>
+#include "arts_export.h"
 
 /*
  * BC - Status (2002-03-08): Dispatcher
@@ -58,7 +59,7 @@ extern "C" {
   typedef void (*mcop_sighandler)(int); 
 }
 
-class Dispatcher {
+class ARTS_EXPORT Dispatcher {
 private:
 	class DispatcherPrivate *d;
 
