@@ -20,7 +20,7 @@
 
     */
 
-#include "core.h"
+#include "common.h"
 
 using namespace Arts;
 
