@@ -33,9 +33,11 @@
 #include <string>
 #include <vector>
 
+#include "arts_export.h"
+
 namespace Arts {
 
-class MCOPUtils {
+class ARTS_EXPORT MCOPUtils {
 public:
 	/**
 	 * Returns the full pathname to a file in the mcop directory which
