@@ -23,6 +23,7 @@
 
     */
 
+#include <cstdlib>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
@@ -122,12 +123,12 @@ void startInclude( const char *line )
 		char *b = strtok(0,">\"");
 		if(b) file = b;
 	}
-	free(l);
 
 	idl_in_include++;
 	idl_include_stack.push(make_pair(idl_line_no, idl_filename));
 	idl_line_no = 0;
 	idl_filename =  file;
+	free(l);
 }
 
 void endInclude()
@@ -729,6 +730,7 @@ FILE *startHeader(string prefix)
 	fprintf(header,"#ifndef %s\n",mkdef(prefix).c_str());
 	fprintf(header,"#define %s\n\n",mkdef(prefix).c_str());
 	fprintf(header,"#include \"common.h\"\n\n");
+	fprintf(header,"#include \"arts_export.h\"\n\n");
 	
 	list<string>::iterator cii;
 	for(cii=customIncludes.begin(); cii != customIncludes.end(); cii++)
@@ -795,8 +797,8 @@ void moveIfChanged(string base)
 		}
 		if (olen < blen) break;
 	}
-	delete newb;
-	delete oldb;
+	delete [] newb;
+	delete [] oldb;
 	fclose(newf);
 	fclose(oldf);
 	if (different) {
@@ -885,7 +887,7 @@ void doStructHeader(FILE *header)
 		nspace.setFromSymbol(d.name.c_str());
 		string tname = nspace.printableForm(d.name);
 
-		fprintf(header,"class %s : public Arts::Type {\n",tname.c_str());
+		fprintf(header,"class ARTS_EXPORT %s : public Arts::Type {\n",tname.c_str());
 		fprintf(header,"public:\n");
 
 		/** constructor without arguments **/
@@ -1308,7 +1310,7 @@ void checkSymbolDefinition(const string& name, const string& type,
 	string xwhere = where.name + "::" + name + " ("+type+")";
 	string& mapentry = defs[name];
 
-	if(mapentry == "")
+	if(mapentry.empty())
 	{
 		mapentry = xwhere;
 	}
@@ -1353,12 +1355,12 @@ void doInterfacesHeader(FILE *header)
 
 		// create abstract interface
 		inherits = buildInheritanceList(d,"_base");
-		if(inherits == "") inherits = "virtual public Arts::Object_base";
+		if(inherits.empty()) inherits = "virtual public Arts::Object_base";
 
 		nspace.setFromSymbol(d.name);
 		iname = nspace.printableForm(d.name);
 
-		fprintf(header,"class %s_base : %s {\n",iname.c_str(),inherits.c_str());
+		fprintf(header,"class ARTS_EXPORT %s_base : %s {\n",iname.c_str(),inherits.c_str());
 		fprintf(header,"public:\n");
 		fprintf(header,"\tstatic unsigned long _IID; // interface ID\n\n");
 		fprintf(header,"\tstatic %s_base *_create(const std::string& subClass"
@@ -1423,9 +1425,9 @@ void doInterfacesHeader(FILE *header)
 		// create stub
 
 		inherits = buildInheritanceList(d,"_stub");
-		if(inherits == "") inherits = "virtual public Arts::Object_stub";
+		if(inherits.empty()) inherits = "virtual public Arts::Object_stub";
 
-		fprintf(header,"class %s_stub : virtual public %s_base, %s {\n",
+		fprintf(header,"class ARTS_EXPORT %s_stub : virtual public %s_base, %s {\n",
 			iname.c_str(), iname.c_str(),inherits.c_str());
 		fprintf(header,"protected:\n");
 		fprintf(header,"\t%s_stub();\n\n",iname.c_str());
@@ -1469,9 +1471,9 @@ void doInterfacesHeader(FILE *header)
 		// create skeleton
 
 		inherits = buildInheritanceList(d,"_skel");
-		if(inherits == "") inherits = "virtual public Arts::Object_skel";
+		if(inherits.empty()) inherits = "virtual public Arts::Object_skel";
 
-		fprintf(header,"class %s_skel : virtual public %s_base,"
+		fprintf(header,"class ARTS_EXPORT %s_skel : virtual public %s_base,"
 			" %s {\n",iname.c_str(),iname.c_str(),inherits.c_str());
 
 		bool firstStream = true;
@@ -1611,7 +1613,7 @@ void doInterfacesHeader(FILE *header)
 
 		inherits = ": public Arts::Object";
 
-		fprintf(header,"class %s %s {\n",iname.c_str(),inherits.c_str());
+		fprintf(header,"class ARTS_EXPORT %s %s {\n",iname.c_str(),inherits.c_str());
 		fprintf(header,"private:\n");
 		fprintf(header,"\tstatic Arts::Object_base* _Creator();\n");
 		fprintf(header,"\t%s_base *_cache;\n",iname.c_str());
