@@ -102,30 +102,13 @@ namespace Arts {
 
 class HandleNotifications : public TimeNotify {
 public:
-	HandleNotifications()
-	{
-		Arts::Dispatcher::the()->ioManager()->addTimer(50, this);
-	}
 	void notifyTime()
-	{
-		NotificationManager::the()->run();
-	}
-	virtual ~HandleNotifications()
 	{
 		Arts::Dispatcher::the()->ioManager()->removeTimer(this);
+		NotificationManager::the()->run();
+		delete this;
 	}
 };
-
-class HandleNotificationsStartup :public StartupClass
-{
-public:
-	void startup()	{ h = new HandleNotifications(); }
-	void shutdown()	{ delete h; }
-private:
-	HandleNotifications *h;
-};
-static HandleNotificationsStartup handleNotifications;
-
 }
 
 /*
@@ -237,6 +220,14 @@ void QIOManager::remove(IONotify *notify, int types)
 
 void QIOManager::addTimer(int milliseconds, TimeNotify *notify)
 {
+	if (milliseconds == -1 && notify == 0)
+	{
+		// HACK: in order to not add a virtual function to IOManager we're calling addTimer with
+		// magic values. This call tells the ioManager that notifications are pending and
+		// NotificationManager::run() should get called soon.
+		notify = new HandleNotifications();
+		milliseconds = 0;
+	}
 	timeList.push_back(new QTimeWatch(milliseconds,notify));
 }
 
